"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const migration_1 = require("./migration");
const utils_1 = require("./utils");
const handlers_1 = require("./handlers");
exports.up = handlers_1.up;
exports.down = handlers_1.down;
class ServerlessTypeOrmMigration {
    constructor(serverless, options) {
        this.serverless = serverless;
        this.options = options;
        this.log = (message) => this.serverless.cli.log(`Serverless Migrations - ${message}`);
        this.commands = {
            migrate: {
                usage: "",
                lifecycleEvents: ["help"],
                commands: {
                    up: {
                        usage: "Runs forward migrations",
                        lifecycleEvents: ["migrate"],
                    },
                    down: {
                        usage: "Rolls back migration",
                        lifecycleEvents: ["rollback"],
                    },
                },
            },
        };
        this.hooks = {
            "migrate:help": this.missingArguments.bind(this),
            "migrate:up:migrate": this.migrate.bind(this),
            "migrate:down:rollback": this.rollback.bind(this),
        };
    }
    missingArguments() {
        this.log(`Missing arguments run serverless migrate --help`);
    }
    afterDeploy() {
        if (this.options.function) {
            this.log(`Calling migration function: ${this.options.function}`);
            this.serverless.pluginManager.invoke(["invoke"]);
        }
        else {
            this.log("No migration function defined");
            this.log("Specify a function name using the --function / -f option.");
        }
    }
    migrate() {
        return __awaiter(this, void 0, void 0, function* () {
            this.log("Looking for migrations");
            const migration = new migration_1.default({
                type: utils_1.getEngine(this.log),
                url: utils_1.getDatabaseConnectionString(this.log),
                migrations: [utils_1.getMigrationFolder(this.log)],
            });
            const migrations = yield migration.runMigration();
            utils_1.logMigrations(this.log, migrations);
        });
    }
    rollback() {
        return __awaiter(this, void 0, void 0, function* () {
            this.log("Undoing last migration");
            const migration = new migration_1.default({
                type: utils_1.getEngine(this.log),
                url: utils_1.getDatabaseConnectionString(this.log),
                migrations: [utils_1.getMigrationFolder(this.log)],
            });
            yield migration.undoLastMigration();
            this.log("Done.");
        });
    }
}
exports.default = ServerlessTypeOrmMigration;
